"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.QueryBuilder = void 0;
const vue_1 = require("vue");
class QueryBuilder {
    constructor(queryKeys, path) {
        this._filters = {};
        this._pageNumber = null;
        this._pageSize = null;
        this._sort = null;
        this._append = (0, vue_1.ref)([]);
        this._include = (0, vue_1.ref)([]);
        this._body = (0, vue_1.ref)({});
        if ((0, vue_1.isRef)(queryKeys)) {
            this._keys = queryKeys;
        }
        else {
            this._keys = (0, vue_1.ref)(queryKeys);
        }
        if ((0, vue_1.isRef)(path)) {
            this._path = path;
        }
        else {
            this._path = (0, vue_1.ref)(path);
        }
    }
    get hasPageNumber() {
        return this._pageNumber != null;
    }
    get hasPageSize() {
        return this._pageSize != null;
    }
    get hasSort() {
        return this._sort != null;
    }
    get hasFilters() {
        return Object.keys(this._filters).length > 0;
    }
    get hasAppend() {
        return this._append.value.length > 0;
    }
    get hasInclude() {
        return this._include.value.length > 0;
    }
    get hasBody() {
        return !!this._body;
    }
    get params() {
        return (0, vue_1.computed)(() => {
            var _a, _b, _c;
            const params = {};
            if (this.hasFilters) {
                Object.entries(this._filters).forEach(([key, value]) => {
                    params[key] = value.value;
                });
            }
            if (this.hasPageSize) {
                params["page[size]"] = (_a = this._pageSize) === null || _a === void 0 ? void 0 : _a.value;
            }
            if (this.hasPageNumber) {
                params["page[number]"] = (_b = this._pageNumber) === null || _b === void 0 ? void 0 : _b.value;
            }
            if (this.hasSort) {
                params.sort = (_c = this._sort) === null || _c === void 0 ? void 0 : _c.value;
            }
            if (this.hasAppend) {
                params.append = this._append.value;
            }
            if (this.hasInclude) {
                params.include = this._include.value;
            }
            return params;
        });
    }
    get path() {
        return this._path;
    }
    get body() {
        return this._body;
    }
    set body(body) {
        if (body === undefined)
            return;
        if ((0, vue_1.isRef)(body)) {
            this._body = body;
        }
        else {
            this._body = (0, vue_1.ref)(body);
        }
    }
    get keys() {
        return (0, vue_1.computed)(() => {
            var _a, _b, _c;
            const mainQueryKeys = this._keys.value;
            // If any of the query keys contain an object, this is an error and should be caught
            // Catch this by looking for `[object Object]` in a key string
            if (mainQueryKeys.some((key) => key.toString().includes("[object Object]"))) {
                throw new Error("Query keys should not contain objects, only strings");
            }
            let keys = [...mainQueryKeys];
            if (this.hasFilters) {
                const filters = Object.entries(this._filters).map(([key, value]) => [key, value.value]);
                keys = [...keys, ...filters];
            }
            if (this.hasPageSize) {
                keys = [...keys, ["page[size]", (_a = this._pageSize) === null || _a === void 0 ? void 0 : _a.value]];
            }
            if (this.hasPageNumber) {
                keys = [...keys, ["page[number]", (_b = this._pageNumber) === null || _b === void 0 ? void 0 : _b.value]];
            }
            if (this.hasSort) {
                keys = [...keys, ["sort", (_c = this._sort) === null || _c === void 0 ? void 0 : _c.value]];
            }
            if (this.hasAppend) {
                keys = [...keys, ["append", this._append.value]];
            }
            if (this.hasInclude) {
                keys = [...keys, ["include", this._include.value]];
            }
            return keys;
        });
    }
    static keys(queryKey) {
        return queryKey;
    }
    static key(queryKey) {
        return queryKey;
    }
    filter(key, value) {
        if (value === undefined)
            return;
        if (value === null)
            return;
        if ((0, vue_1.isRef)(value)) {
            this._filters[`filter[${key}]`] = value;
            return this;
        }
        else {
            this._filters[`filter[${key}]`] = (0, vue_1.ref)(value);
            return this;
        }
    }
    filterIf(condition, key, value) {
        if (!condition)
            return;
        this.filter(key, value);
    }
    sort(sort) {
        if (sort === undefined)
            return;
        if ((0, vue_1.isRef)(sort))
            this._sort = sort;
        else
            this._sort = (0, vue_1.ref)(sort);
    }
    page(page) {
        if (page === undefined)
            return;
        if ((0, vue_1.isRef)(page))
            this._pageNumber = page;
        else
            this._pageNumber = (0, vue_1.ref)(page);
    }
    size(size) {
        if (size === undefined)
            return;
        if ((0, vue_1.isRef)(size))
            this._pageSize = size;
        else
            this._pageSize = (0, vue_1.ref)(size);
    }
    withKey(key) {
        if (key === undefined)
            return;
        this._keys.value.push((0, vue_1.isRef)(key) ? key.value : key);
    }
    withKeys(key) {
        if (key === undefined)
            return;
        this._keys.value.push(...((0, vue_1.isRef)(key) ? key.value : key));
    }
    withKeyIf(condition, key) {
        if (!condition)
            return;
        this.withKey(key);
    }
    withKeysIf(condition, key) {
        if (!condition)
            return;
        this.withKeys(key);
    }
    append(append) {
        if (append === undefined)
            return;
        if ((0, vue_1.isRef)(append)) {
            if (Array.isArray(append.value)) {
                this._append.value.push(...append.value);
            }
            else {
                this._append.value.push(append.value);
            }
        }
        else {
            if (Array.isArray(append)) {
                this._append.value.push(...append);
            }
            else {
                this._append.value.push(append);
            }
        }
    }
    include(include) {
        if (include === undefined)
            return;
        if ((0, vue_1.isRef)(include)) {
            if (Array.isArray(include.value)) {
                this._include.value.push(...include.value);
            }
            else {
                this._include.value.push(include.value);
            }
        }
        else {
            if (Array.isArray(include)) {
                this._include.value.push(...include);
            }
            else {
                this._include.value.push(include);
            }
        }
    }
    withBody(body) {
        if ((0, vue_1.isRef)(body)) {
            this._body = body;
        }
        else {
            this._body = (0, vue_1.ref)(body);
        }
    }
    /**
     * Alias for withBody
     */
    withPayload(payload) {
        this.withBody(payload);
    }
    /**
     * Build the query params for the query.
     */
    build(params) {
        this.size(params.size);
        this.withKeys(params.additionalKeys);
        this.page(params.page);
        this.append(params.append);
        this.include(params.include);
        this.sort(params.sort);
        if (params.body)
            this.withBody(params.body);
        Object.entries(((0, vue_1.isRef)(params.filters)
            ? params.filters.value
            : params.filters) || {}).forEach(([key, value]) => {
            this.filter(key, value);
        });
    }
    static mutate(path) {
        return new QueryBuilder([], path);
    }
}
exports.QueryBuilder = QueryBuilder;
