import { useAxios } from "./useAxios";
import { Ref, computed, ref, watch } from "vue";
import { BaseQueryKey, QueryBuilder } from "./QueryBuilder";
import { useInfiniteQuery, useQuery } from "@tanstack/vue-query";
import { ApiPaginatedGETResponse, PaginatedData } from "./PaginatedData";

export class Query<QueryKey extends BaseQueryKey, T, Model> {
  queryBuilder: QueryBuilder<QueryKey>;

  constructor(queryBuilder: QueryBuilder<QueryKey>) {
    this.queryBuilder = queryBuilder;
  }

  getModel({
    select,
    dependsOn = [],
  }: {
    select: (data: T) => Model;
    dependsOn?: Ref<any | undefined | null>[] | undefined;
  }) {
    const { axios } = useAxios();

    return useQuery({
      queryKey: this.queryBuilder.keys,
      queryFn: () =>
        axios
          .get<T>(this.queryBuilder.path.value, {
            params: this.queryBuilder.params.value,
          })
          .then((data) => data.data),

      select: select,

      enabled: computed(() =>
        dependsOn?.every((ref) => ref.value !== null && ref.value !== undefined)
      ),
    });
  }

  getPaginatedModels({
    select,
    dependsOn = [],
  }: {
    select: (data: T) => Model;
    dependsOn?: Ref<any>[] | undefined;
  }) {
    const { axios } = useAxios();

    const total = ref<number | undefined>(undefined);

    const query = useInfiniteQuery({
      queryKey: this.queryBuilder.keys,

      queryFn: ({ pageParam = 1 }) => {
        return axios
          .get<ApiPaginatedGETResponse<T>>(this.queryBuilder.path.value, {
            params: {
              "page[number]": pageParam,
              ...this.queryBuilder.params.value,
            },
          })
          .then((resp) => {
            total.value = resp.data.total;
            return resp.data;
          });
      },

      initialPageParam: 1,

      getNextPageParam: (lastPage, _pages) =>
        lastPage.next_page_url ? lastPage.current_page + 1 : undefined,

      select: ({ pageParams, pages }) => {
        total.value = pages[0]?.total;

        const transformedPages = pages.map((data) => {
          return new PaginatedData<Model, T>(data, select);
        });

        return { pages: transformedPages, pageParams };
      },

      enabled: computed(() =>
        dependsOn?.every((ref) => ref.value !== null && ref.value !== undefined)
      ),
    });

    const data = computed<PaginatedData<Model, T>[] | undefined>(() =>
      query.data.value?.pages.flat()
    );

    const models = computed<Model[] | undefined>(() =>
      data.value?.map((item) => item.data).flat()
    );

    return {
      ...query,
      data,
      models,
      total,
    };
  }

  getAllModels({
    select,
    dependsOn = [],
  }: {
    select: (data: T) => Model;
    dependsOn?: Ref<any>[] | undefined;
  }) {
    const isLoadingLocal = ref(true);

    const results = ref<Model[]>();

    const { data, hasNextPage, fetchNextPage, isRefetching, total, refetch } =
      this.getPaginatedModels({ select, dependsOn });

    watch(
      [hasNextPage, computed(() => data.value?.length), isRefetching],
      () => {
        if (hasNextPage?.value && !isRefetching?.value) {
          fetchNextPage();
        }

        if (!hasNextPage?.value && !isRefetching?.value) {
          isLoadingLocal.value = false;
          results.value = data.value?.flatMap((item) => item.data);
        }
      }
    );

    return {
      data: computed(() => results.value ?? []),
      isLoading: isLoadingLocal,
      isRefetching,
      total,
      refetch,
    };
  }
}
